class DriverCalc:
    def __init__(self, source_dict, columns, models):
        import pandas as pd
        from flask import current_app
        self.pd = pd
        self.complete_dataframe = pd.DataFrame(source_dict, columns=columns)
        self.models = models
        self.output_path = current_app.config['UPLOAD_FOLDER']
        self.file_path = ""
        self.column_map = {
            "Date": 0,
            "Driver": 1,
            "Restaurant Name": 2,
            "Receipt #": 3,
            "Customer Address": 4,
            "Driver Paid Amount": 5,
            # Pre-tax total
            "Total Restaurant": 6,
            "Tips": 7,
            "Customer payment type": 8,
            "Additional Pick-Up": 9,
            # Post-tax total
            "post_tax_total": 10,
            "crave_fee": 11,
            "driver_fee": 12,
            "ad_crave_fee": 13,
            "ad_driver_fee": 14,
            "service_charge": 15,
            "customer_paid": 16,
            "cash_collected": 17,
            "owed_to_driver": 18
        }
        self.calc()

    def calc(self):
        complete_dataframe = self.complete_dataframe

        post_tax_total = complete_dataframe.apply(self.post_tax_total, axis=1)
        post_tax_total.name = "post_tax_total"
        complete_dataframe.update(post_tax_total)

        crave_fee = complete_dataframe.apply(self.crave_fee, axis=1)
        crave_fee.name = "crave_fee"
        complete_dataframe.update(crave_fee)

        driver_fee = complete_dataframe.apply(self.driver_fee, axis=1)
        driver_fee.name = "driver_fee"
        complete_dataframe.update(driver_fee)

        ad_crave_fee = complete_dataframe.apply(self.ad_crave_fee, axis=1)
        ad_crave_fee.name = "ad_crave_fee"
        complete_dataframe.update(ad_crave_fee)

        ad_driver_fee = complete_dataframe.apply(self.ad_driver_fee, axis=1)
        ad_driver_fee.name = "ad_driver_fee"
        complete_dataframe.update(ad_driver_fee)

        service_charge = complete_dataframe.apply(self.service_charge, axis=1)
        service_charge.name = "service_charge"
        complete_dataframe.update(service_charge)

        customer_paid = complete_dataframe.apply(self.customer_paid, axis=1)
        customer_paid.name = "customer_paid"
        complete_dataframe.update(customer_paid)

        cash_collected = complete_dataframe.apply(self.cash_collected, axis=1)
        cash_collected.name = "cash_collected"
        complete_dataframe.update(cash_collected)

        owed_to_driver = complete_dataframe.apply(self.owed_to_driver, axis=1)
        owed_to_driver.name = "owed_to_driver"
        complete_dataframe.update(owed_to_driver)

        date_string = complete_dataframe.apply(self.date_string, axis=1)
        date_string.name = "date_string"
        complete_dataframe.update(date_string)

        complete_dataframe = complete_dataframe.dropna(axis=0, how="all", subset=["Driver"])

        self.complete_dataframe.update(complete_dataframe)

    def post_tax_total(self, df):
        restaurant = self.models.query.filter_by(name=df["Restaurant"]).first()
        return (restaurant.tax() + 1) * df["Total Restaurant"]

    def crave_fee(self, df):
        if df["Additional Pick-Up"] == False:
            restaurant = self.models.query.filter_by(name=df["Restaurant"]).first()
            return restaurant.crave_fee
        else:
            return 0

    def driver_fee(self, df):
        if df["Additional Pick-Up"] == False:
            restaurant = self.models.query.filter_by(name=df["Restaurant"]).first()
            return restaurant.driver_fee
        else:
            return 0

    def ad_crave_fee(self, df):
        if df["Additional Pick-Up"] == True:
            restaurant = self.models.query.filter_by(name=df["Restaurant"]).first()
            return restaurant.additional_crave_fee
        else:
            return 0

    def ad_driver_fee(self, df):
        if df["Additional Pick-Up"] == True:
            restaurant = self.models.query.filter_by(name=df["Restaurant"]).first()
            return restaurant.additional_driver_fee
        else:
            return 0

    def service_charge(self, df):
        if df["Customer payment type"] != "Pre-paid":
            restaurant = self.models.query.filter_by(name=df["Restaurant"]).first()
            return restaurant.service_fee * df["Total Restaurant"]
        else:
            return 0

    def cash_collected(self, df):
        if df["Customer payment type"] == "Cash":
            cash = df["post_tax_total"] + \
                   df["crave_fee"] + \
                   df["driver_fee"] + \
                   df["ad_crave_fee"] + \
                   df["ad_driver_fee"] + \
                   df["service_charge"] + \
                   df["Tips"]

        elif df["Customer payment type"] == "Pre-paid":
            cash = df["crave_fee"] + \
                   df["driver_fee"] + \
                   df["ad_crave_fee"] + \
                   df["ad_driver_fee"] + \
                   df["service_charge"] + \
                   df["Tips"]

        else:
            cash = 0
        return cash

    def date_string(self, df):
        date = df["Date"]
        dt = self.pd.to_datetime(date, errors='coerce')
        try:
            datetime = dt.strftime("%m-%d-%Y")
        except:
            datetime = False

        if datetime:
            return datetime

    def customer_paid(self, df):
        result = df["post_tax_total"] + \
                 df["crave_fee"] + \
                 df["driver_fee"] + \
                 df["ad_crave_fee"] + \
                 df["ad_driver_fee"] + \
                 df["service_charge"] + \
                 df["Tips"]
        return result

    def owed_to_driver(self, df):
        driver_owes = df["driver_fee"] + \
                      df["ad_driver_fee"] + \
                      df["Tips"] + \
                      df["Driver Paid Amount"] - \
                      df["cash_collected"]

        return driver_owes

    def report_driver(self, report_name):
        driver_df = self.complete_dataframe

        renamed_columns = {
            "Date": "Date",
            "Driver": "Driver",
            "Restaurant Name": "Restaurant Name",
            "Receipt #": "Receipt #",
            "Customer Address": "Customer Address",
            "Driver Paid Amount": "Driver Paid Amount",
            "Total Restaurant": "Total Restaurant",
            "Tips": "Tips",
            "Customer payment type": "Payment Type",
            "Additional Pick-Up": "Additional Pick-Up",
            "post_tax_total": "Post-tax Total",
            "crave_fee": "Crave Fee",
            "driver_fee": "Driver Fee",
            "ad_crave_fee": "Additional Crave Fee",
            "ad_driver_fee": "Additional Driver Fee",
            "service_charge": "Service Fee",
            "customer_paid": "Customer Paid",
            "cash_collected": "Cash Collected",
            "owed_to_driver": "Driver Balance"
        }
        self.file_path = "{}{}.xlsx".format(self.output_path, report_name)
        print(f"Creating report: {self.file_path}")
        with self.pd.ExcelWriter(self.file_path, mode="w", engine='xlsxwriter') as writer:

            numbered_columns = [driver_df.columns[i] for i in [5, 6, 7, 10, 11, 12, 13, 14, 15, 16, 17, 18]]
            for col in numbered_columns:
                driver_df[col] = driver_df[col].astype(float)

            driver_df.to_excel(writer, index=False, sheet_name="Report")

            number_rows = len(driver_df.dropna(axis=0, how="all", subset=["Driver"]).index)

            workbook = writer.book
            report_worksheet = writer.sheets["Report"]

            header_format = workbook.add_format({
                'bold': True,
                'valign': 'top',
                'fg_color': '#D7E4BC',
                'border': 1})

            currency_format = workbook.add_format({'num_format': '$#,##0.00'})

            # Add currency format to money columns
            report_worksheet.set_column("F:F", 5, cell_format=currency_format)
            report_worksheet.set_column("G:G", 6, cell_format=currency_format)
            report_worksheet.set_column("H:H", 7, cell_format=currency_format)
            report_worksheet.set_column("I:I", 8, cell_format=currency_format)
            report_worksheet.set_column("K:K", 8, cell_format=currency_format)
            report_worksheet.set_column("L:L", 11, cell_format=currency_format)
            report_worksheet.set_column("M:M", 12, cell_format=currency_format)
            report_worksheet.set_column("N:N", 13, cell_format=currency_format)
            report_worksheet.set_column("O:O", 14, cell_format=currency_format)
            report_worksheet.set_column("P:P", 15, cell_format=currency_format)
            report_worksheet.set_column("Q:Q", 16, cell_format=currency_format)
            report_worksheet.set_column("R:R", 17, cell_format=currency_format)
            report_worksheet.set_column("S:S", 18, cell_format=currency_format)

            # Reformating Column headers
            for column in renamed_columns.keys():
                column_num = self.column_map[column]
                new_name = renamed_columns[column]
                report_worksheet.write(0, column_num, new_name, header_format)

            subtotal_columns = {
                "Driver Paid Amount": driver_df["Driver Paid Amount"].sum(),
                "Total Restaurant": driver_df["Total Restaurant"].sum(),
                "post_tax_total": driver_df["post_tax_total"].sum(),
                "Tips": driver_df["Tips"].sum(),
                "crave_fee": driver_df["crave_fee"].sum(),
                "driver_fee": driver_df["driver_fee"].sum(),
                "ad_crave_fee": driver_df["ad_crave_fee"].sum(),
                "ad_driver_fee": driver_df["ad_driver_fee"].sum(),
                "service_charge": driver_df["service_charge"].sum(),
                "customer_paid": driver_df["customer_paid"].sum(),
                "cash_collected": driver_df["cash_collected"].sum(),
                "owed_to_driver": driver_df["owed_to_driver"].sum()
            }

            # Subtotals
            for column in ["Driver Paid Amount", "Total Restaurant", "post_tax_total", "Tips",
                           "crave_fee", "driver_fee", "ad_crave_fee", "ad_driver_fee",
                           "customer_paid", "service_charge", "cash_collected",
                           "owed_to_driver"]:

                column_num = self.column_map[column]

                report_worksheet.write(
                    number_rows + 1, column_num, "Subtotal - {}".format(renamed_columns[column]), header_format
                )

                report_worksheet.write(
                    number_rows + 2, column_num, subtotal_columns[column]
                )

            # Totals
            ## Title (Totals) + 4
            report_worksheet.write(number_rows + 4, 0, "Totals", header_format)

            ## Driver Paid +5
            report_worksheet.write(number_rows + 5, 0, "Total Driver Paid")
            report_worksheet.write(number_rows + 5, 1, driver_df["Driver Paid Amount"].sum(), currency_format)

            ## Total Received From Customers +6
            report_worksheet.write(number_rows + 6, 0, "Total Received From Customers")
            report_worksheet.write(number_rows + 6, 1, driver_df["customer_paid"].sum(), currency_format)

            ## Title (Deductions) +8
            report_worksheet.write(number_rows + 8, 0, "Deductions", header_format)

            # Cash Collected from Customers +9
            report_worksheet.write(number_rows + 9, 0, "Total Cash Collected from Customers")
            report_worksheet.write(number_rows + 9, 1, driver_df["cash_collected"].sum(), currency_format)

            # Total Crave Fees +10
            total_crave_fees = driver_df["crave_fee"].sum() + driver_df["ad_crave_fee"].sum()
            report_worksheet.write(number_rows + 10, 0, "Total Crave Fees")
            report_worksheet.write(number_rows + 10, 1, total_crave_fees, currency_format)

            # Total Service Fees +11
            report_worksheet.write(number_rows + 11, 0, "Total Service Fees")
            report_worksheet.write(number_rows + 11, 1, driver_df["service_charge"].sum(), currency_format)

            # Total Restaurant Bills  +12
            report_worksheet.write(number_rows + 12, 0, "Total Restaurant Bills")
            report_worksheet.write(number_rows + 12, 1, driver_df["Total Restaurant"].sum(), currency_format)

            # Total Bills After Taxes  +12
            report_worksheet.write(number_rows + 12, 0, "Total After-tax Bills")
            report_worksheet.write(number_rows + 12, 1, driver_df["Total Restaurant"].sum(), currency_format)

            # Driver Balance ( + for crave owes driver, (-) for driver owes crave) +13
            report_worksheet.write(number_rows + 13, 0, "Driver Balance ( + for crave owes driver, (-) for driver owes crave)", header_format)
            report_worksheet.write(number_rows + 13, 1, driver_df["owed_to_driver"].sum(), currency_format)

            # Driver Balance ( + for crave owes driver, (-) for driver owes crave) +13
            report_worksheet.write(number_rows + 14, 0, "Driver Takehome", header_format)
            takehome = driver_df["Tips"].sum() + driver_df["driver_fee"].sum() + driver_df["ad_driver_fee"].sum()
            report_worksheet.write(number_rows + 14, 1, takehome, currency_format)

            writer.save()


class RestaurantCalc:
    def __init__(self, source_dict, columns, models, delivery_rebate, commission, monthly_fee):
        import pandas as pd
        from config import Config
        self.pd = pd
        self.complete_dataframe = pd.DataFrame(source_dict, columns=columns)
        self.models = models
        self.output_path = Config.UPLOAD_FOLDER
        self.delivery_rebate = delivery_rebate
        self.commission = commission
        self.monthly_fee = monthly_fee


    def report_restaurant_weekly(self, report_name, week_ending):
        from datetime import datetime
        restaurant_df = self.complete_dataframe
        rest_df = restaurant_df.dropna(axis=0, how="all", subset=["Driver"])
        dates_dfs = dict()
        days_in_df = rest_df["Date"].unique()
        for day in days_in_df:
            dates_dfs[day] = rest_df.where(rest_df["Date"] == day).dropna(axis=0, how="all", subset=["Driver"])

        with self.pd.ExcelWriter(
                "{}{}.xlsx".format(self.output_path, report_name), mode="w", engine='xlsxwriter'
        ) as writer:

            numbered_column = rest_df.columns[3]
            rest_df[numbered_column] = rest_df[numbered_column].astype(float)
            workbook = writer.book
            report_worksheet = workbook.add_worksheet("Report")
            num_rows = 0

            header_format = workbook.add_format({
                'bold': True,
                'valign': 'top',
                'fg_color': '#D7E4BC',
                'border': 1
            })

            currency_format = workbook.add_format({'num_format': '$#,##0.00'})

            report_worksheet.write(0, 0, "Week Ending {}".format(datetime.strftime(week_ending, "%m/%d/%Y")), header_format)

            weekly_total = 0
            weekly_commission_total = 0
            weekly_rebate_total = 0
            daily_commission_total = 0
            daily_rebate_total = 0

            for day in dates_dfs.keys():
                deductions_total = 0
                daily_commission_total = 0
                daily_rebate_total = 0

                report_worksheet.write(num_rows + 2, 0, "Day - {}".format(day), header_format)

                report_worksheet.write(num_rows + 3, 0, "Driver Name", header_format)
                report_worksheet.write(num_rows+3, 1, "Date", header_format)
                report_worksheet.write(num_rows+3, 2, "Receipt #", header_format)
                report_worksheet.write(num_rows+3, 3, "Pre-tax Food Total", header_format)

                if self.commission == 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows+3, 4, "Delivery Subsidy", header_format)
                    report_worksheet.write(num_rows + 3, 5, "Deductions Subtotal", header_format)

                elif self.commission > 0 and self.delivery_rebate == 0:
                    report_worksheet.write(num_rows + 3, 4, "Crave Commission", header_format)
                    report_worksheet.write(num_rows + 3, 5, "Deductions Subtotal", header_format)

                elif self.commission > 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows + 3, 4, "Delivery Subsidy", header_format)
                    report_worksheet.write(num_rows + 3, 5, "Crave Commission", header_format)
                    report_worksheet.write(num_rows + 3, 6, "Deductions Subtotal", header_format)

                report_worksheet.write_column(num_rows + 4, 0, dates_dfs[day]["Driver"])
                report_worksheet.write_column(num_rows+4, 1, dates_dfs[day]["Date"])
                report_worksheet.write_column(num_rows+4, 2, dates_dfs[day]["Receipt #"])
                report_worksheet.write_column(num_rows+4, 3, dates_dfs[day]["Total Restaurant"], currency_format)



                if self.commission > 0 and self.delivery_rebate == 0:
                    deductions_subtotal = dates_dfs[day]["Total Restaurant"] * self.commission
                    report_worksheet.write_column(num_rows + 4, 4, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    report_worksheet.write_column(num_rows + 4, 5, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    daily_commission_total += deductions_subtotal.sum()
                    num_rows += 2

                elif self.commission == 0 and self.delivery_rebate > 0:
                    deductions_subtotal = self.delivery_rebate * 1
                    report_worksheet.write_column(num_rows + 4, 4, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    report_worksheet.write_column(num_rows + 4, 5, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    daily_rebate_total += self.delivery_rebate * len(dates_dfs[day]["Total Restaurant"].index)
                    num_rows += 2

                elif self.commission > 0 and self.delivery_rebate > 0:
                    commission_total = dates_dfs[day]["Total Restaurant"] * self.commission
                    deductions_subtotal = commission_total + self.delivery_rebate
                    report_worksheet.write_column(num_rows + 4, 4, [self.delivery_rebate for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    report_worksheet.write_column(num_rows + 4, 5, commission_total, currency_format)
                    report_worksheet.write_column(num_rows + 4, 6, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    daily_commission_total += deductions_subtotal.sum()
                    daily_rebate_total += self.delivery_rebate * len(dates_dfs[day]["Total Restaurant"].index)
                    num_rows += 3

                num_rows += len(dates_dfs[day].dropna(axis=0, how="all", subset=["Driver"]).index) + 1
                deductions_total = daily_commission_total + daily_rebate_total

                if self.commission > 0 and self.delivery_rebate == 0:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)

                    report_worksheet.write(num_rows + 3, 0, "Daily Commission Total:", header_format)
                    report_worksheet.write(num_rows + 3, 1, daily_commission_total, currency_format)

                    report_worksheet.write(num_rows + 4, 0, "Daily Deductions Total:", header_format)
                    report_worksheet.write(num_rows + 4, 1, deductions_total, currency_format)

                    report_worksheet.write(num_rows + 5, 0, "Daily Total:", header_format)
                    report_worksheet.write(num_rows + 5, 1, (dates_dfs[day]["Total Restaurant"].sum() - deductions_total), currency_format)

                    weekly_total += (dates_dfs[day]["Total Restaurant"].sum() - deductions_total)
                    num_rows += 5

                elif self.commission == 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)

                    report_worksheet.write(num_rows + 3, 0, "Daily Subsidy Total:", header_format)
                    report_worksheet.write(num_rows + 3, 1, daily_rebate_total, currency_format)

                    report_worksheet.write(num_rows + 4, 0, "Daily Deductions Total:", header_format)
                    report_worksheet.write(num_rows + 4, 1, deductions_total, currency_format)

                    report_worksheet.write(num_rows + 5, 0, "Daily Total:", header_format)
                    report_worksheet.write(num_rows + 5, 1, (dates_dfs[day]["Total Restaurant"].sum() - deductions_total), currency_format)

                    weekly_total += (dates_dfs[day]["Total Restaurant"].sum() - deductions_total)
                    num_rows += 5

                elif self.commission > 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)

                    report_worksheet.write(num_rows + 3, 0, "Daily Commission Total:", header_format)
                    report_worksheet.write(num_rows + 3, 1, daily_commission_total, currency_format)

                    report_worksheet.write(num_rows + 4, 0, "Daily Subsidy Total:", header_format)
                    report_worksheet.write(num_rows + 4, 1, daily_rebate_total, currency_format)

                    report_worksheet.write(num_rows + 5, 0, "Daily Deductions Total:", header_format)
                    report_worksheet.write(num_rows + 5, 1, deductions_total, currency_format)

                    report_worksheet.write(num_rows + 6, 0, "Daily Total:", header_format)
                    report_worksheet.write(num_rows + 6, 1,(dates_dfs[day]["Total Restaurant"].sum() - deductions_total), currency_format)

                    weekly_total += (dates_dfs[day]["Total Restaurant"].sum() - deductions_total)
                    num_rows += 6

                else:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)
                    weekly_total += (dates_dfs[day]["Total Restaurant"].sum())
                    num_rows += 2

            weekly_commission_total += daily_commission_total
            weekly_rebate_total += daily_rebate_total


            report_worksheet.write(num_rows + 2, 0, "Weekly Subtotal:", header_format)
            report_worksheet.write(num_rows + 2, 1, weekly_total, currency_format)
            num_rows += 1

            if self.commission > 0 and self.delivery_rebate == 0:
                report_worksheet.write(num_rows + 1, 0, "Weekly Commission Total:", header_format)
                report_worksheet.write(num_rows + 1, 1, weekly_commission_total, currency_format)
                num_rows += 1

            elif self.commission == 0 and self.delivery_rebate > 0:
                report_worksheet.write(num_rows + 1, 0, "Weekly Subsidy total:", header_format)
                report_worksheet.write(num_rows + 1, 1, weekly_rebate_total, currency_format)
                num_rows += 1

            elif self.commission > 0 and self.delivery_rebate > 0:
                report_worksheet.write(num_rows + 1, 0, "Weekly Commission Total:", header_format)
                report_worksheet.write(num_rows + 1, 1, weekly_commission_total, currency_format)
                report_worksheet.write(num_rows + 2, 0, "Weekly Rebate Total:", header_format)
                report_worksheet.write(num_rows + 2, 1, weekly_rebate_total, currency_format)
                num_rows += 2

            if self.monthly_fee:
                report_worksheet.write(num_rows + 1, 0, "Monthly Fee:", header_format)
                report_worksheet.write(num_rows + 1, 1, self.monthly_fee, currency_format)

                final_total = weekly_total - self.monthly_fee
                report_worksheet.write(num_rows + 2, 0, "Weekly Total:", header_format)
                report_worksheet.write(num_rows + 2, 1, final_total, currency_format)

            else:
                report_worksheet.write(num_rows + 3, 0, "Weekly Total:", header_format)
                report_worksheet.write(num_rows + 3, 1, weekly_total, currency_format)

            writer.save()

    def report_restaurant_monthly(self, report_name, month_of):
        from datetime import datetime
        restaurant_df = self.complete_dataframe
        rest_df = restaurant_df.dropna(axis=0, how="all", subset=["Driver"])
        dates_dfs = dict()
        days_in_df = rest_df["Date"].unique()
        for day in days_in_df:
            dates_dfs[day] = rest_df.where(rest_df["Date"] == day).dropna(axis=0, how="all", subset=["Driver"])

        with self.pd.ExcelWriter(
                "{}{}.xlsx".format(self.output_path, report_name), mode="w", engine='xlsxwriter'
        ) as writer:

            numbered_column = rest_df.columns[3]
            rest_df[numbered_column] = rest_df[numbered_column].astype(float)
            workbook = writer.book
            report_worksheet = workbook.add_worksheet("Report")
            num_rows = 0

            header_format = workbook.add_format({
                'bold': True,
                'valign': 'top',
                'fg_color': '#D7E4BC',
                'border': 1
            })

            currency_format = workbook.add_format({'num_format': '$#,##0.00'})

            report_worksheet.write(0, 0, "Month of {}".format(datetime.strftime(month_of,  "%B, %Y")), header_format)

            monthly_total = 0
            monthly_commission_total = 0
            monthly_rebate_total = 0
            daily_commission_total = 0
            daily_rebate_total = 0

            for day in dates_dfs.keys():
                deductions_total = 0
                daily_commission_total = 0
                daily_rebate_total = 0

                report_worksheet.write(num_rows + 2, 0, "Day - {}".format(day), header_format)

                report_worksheet.write(num_rows + 3, 0, "Driver Name", header_format)
                report_worksheet.write(num_rows+3, 1, "Date", header_format)
                report_worksheet.write(num_rows+3, 2, "Receipt #", header_format)
                report_worksheet.write(num_rows+3, 3, "Pre-tax Food Total", header_format)

                if self.commission == 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows+3, 4, "Delivery Subsidy", header_format)
                    report_worksheet.write(num_rows + 3, 5, "Deductions Subtotal", header_format)

                elif self.commission > 0 and self.delivery_rebate == 0:
                    report_worksheet.write(num_rows + 3, 4, "Crave Commission", header_format)
                    report_worksheet.write(num_rows + 3, 5, "Deductions Subtotal", header_format)

                elif self.commission > 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows + 3, 4, "Delivery Subsidy", header_format)
                    report_worksheet.write(num_rows + 3, 5, "Crave Commission", header_format)
                    report_worksheet.write(num_rows + 3, 6, "Deductions Subtotal", header_format)

                report_worksheet.write_column(num_rows + 4, 0, dates_dfs[day]["Driver"])
                report_worksheet.write_column(num_rows+4, 1, dates_dfs[day]["Date"])
                report_worksheet.write_column(num_rows+4, 2, dates_dfs[day]["Receipt #"])
                report_worksheet.write_column(num_rows+4, 3, dates_dfs[day]["Total Restaurant"], currency_format)



                if self.commission > 0 and self.delivery_rebate == 0:
                    deductions_subtotal = dates_dfs[day]["Total Restaurant"] * self.commission
                    report_worksheet.write_column(num_rows + 4, 4, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    report_worksheet.write_column(num_rows + 4, 5, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    daily_commission_total += deductions_subtotal.sum()
                    num_rows += 2

                elif self.commission == 0 and self.delivery_rebate > 0:
                    deductions_subtotal = self.delivery_rebate * 1
                    report_worksheet.write_column(num_rows + 4, 4, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    report_worksheet.write_column(num_rows + 4, 5, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    daily_rebate_total += self.delivery_rebate * len(dates_dfs[day]["Total Restaurant"].index)
                    num_rows += 2

                elif self.commission > 0 and self.delivery_rebate > 0:
                    commission_total = dates_dfs[day]["Total Restaurant"] * self.commission
                    deductions_subtotal = commission_total + self.delivery_rebate
                    report_worksheet.write_column(num_rows + 4, 4, [self.delivery_rebate for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    report_worksheet.write_column(num_rows + 4, 5, commission_total, currency_format)
                    report_worksheet.write_column(num_rows + 4, 6, [deductions_subtotal for i in dates_dfs[day]["Total Restaurant"]], currency_format)
                    daily_commission_total += deductions_subtotal.sum()
                    daily_rebate_total += self.delivery_rebate * len(dates_dfs[day]["Total Restaurant"].index)
                    num_rows += 3

                num_rows += len(dates_dfs[day].dropna(axis=0, how="all", subset=["Driver"]).index) + 1
                deductions_total = daily_commission_total + daily_rebate_total

                if self.commission > 0 and self.delivery_rebate == 0:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)

                    report_worksheet.write(num_rows + 3, 0, "Daily Commission Total:", header_format)
                    report_worksheet.write(num_rows + 3, 1, daily_commission_total, currency_format)

                    report_worksheet.write(num_rows + 4, 0, "Daily Deductions Total:", header_format)
                    report_worksheet.write(num_rows + 4, 1, deductions_total, currency_format)

                    report_worksheet.write(num_rows + 5, 0, "Daily Total:", header_format)
                    report_worksheet.write(num_rows + 5, 1, (dates_dfs[day]["Total Restaurant"].sum() - deductions_total), currency_format)

                    monthly_total += (dates_dfs[day]["Total Restaurant"].sum() - deductions_total)
                    num_rows += 5

                elif self.commission == 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)

                    report_worksheet.write(num_rows + 3, 0, "Daily Subsidy Total:", header_format)
                    report_worksheet.write(num_rows + 3, 1, daily_rebate_total, currency_format)

                    report_worksheet.write(num_rows + 4, 0, "Daily Deductions Total:", header_format)
                    report_worksheet.write(num_rows + 4, 1, deductions_total, currency_format)

                    report_worksheet.write(num_rows + 5, 0, "Daily Total:", header_format)
                    report_worksheet.write(num_rows + 5, 1, (dates_dfs[day]["Total Restaurant"].sum() - deductions_total), currency_format)

                    monthly_total += (dates_dfs[day]["Total Restaurant"].sum() - deductions_total)
                    num_rows += 5

                elif self.commission > 0 and self.delivery_rebate > 0:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)

                    report_worksheet.write(num_rows + 3, 0, "Daily Commission Total:", header_format)
                    report_worksheet.write(num_rows + 3, 1, daily_commission_total, currency_format)

                    report_worksheet.write(num_rows + 4, 0, "Daily Subsidy Total:", header_format)
                    report_worksheet.write(num_rows + 4, 1, daily_rebate_total, currency_format)

                    report_worksheet.write(num_rows + 5, 0, "Daily Deductions Total:", header_format)
                    report_worksheet.write(num_rows + 5, 1, deductions_total, currency_format)

                    report_worksheet.write(num_rows + 6, 0, "Daily Total:", header_format)
                    report_worksheet.write(num_rows + 6, 1,(dates_dfs[day]["Total Restaurant"].sum() - deductions_total), currency_format)

                    monthly_total += (dates_dfs[day]["Total Restaurant"].sum() - deductions_total)
                    num_rows += 6

                else:
                    report_worksheet.write(num_rows + 2, 0, "Daily Food Total:", header_format)
                    report_worksheet.write(num_rows + 2, 1, dates_dfs[day]["Total Restaurant"].sum(), currency_format)
                    monthly_total += (dates_dfs[day]["Total Restaurant"].sum())
                    num_rows += 2

            monthly_commission_total += daily_commission_total
            monthly_rebate_total += daily_rebate_total

            report_worksheet.write(num_rows + 2, 0, "Monthly Subtotal:", header_format)
            report_worksheet.write(num_rows + 2, 1, monthly_total, currency_format)
            num_rows += 1

            if self.commission > 0 and self.delivery_rebate == 0:
                report_worksheet.write(num_rows + 1, 0, "monthly Commission Total:", header_format)
                report_worksheet.write(num_rows + 1, 1, monthly_commission_total, currency_format)
                num_rows += 1

            elif self.commission == 0 and self.delivery_rebate > 0:
                report_worksheet.write(num_rows + 1, 0, "Monthly Subsidy total:", header_format)
                report_worksheet.write(num_rows + 1, 1, monthly_rebate_total, currency_format)
                num_rows += 1

            elif self.commission > 0 and self.delivery_rebate > 0:
                report_worksheet.write(num_rows + 1, 0, "Monthly Commission Total:", header_format)
                report_worksheet.write(num_rows + 1, 1, monthly_commission_total, currency_format)
                report_worksheet.write(num_rows + 2, 0, "Monthly Rebate Total:", header_format)
                report_worksheet.write(num_rows + 2, 1, monthly_rebate_total, currency_format)
                num_rows += 2

            if self.monthly_fee:
                report_worksheet.write(num_rows + 1, 0, "Monthly Fee:", header_format)
                report_worksheet.write(num_rows + 1, 1, self.monthly_fee, currency_format)

                final_total = monthly_total - self.monthly_fee
                report_worksheet.write(num_rows + 2, 0, "Monthly Total:", header_format)
                report_worksheet.write(num_rows + 2, 1, final_total, currency_format)

            else:
                report_worksheet.write(num_rows + 3, 0, "Monthly Total:", header_format)
                report_worksheet.write(num_rows + 3, 1, monthly_total, currency_format)

            writer.save()

